
;=========================================================;
;                                                         ;
; miniODE example 1: Setup an extremely simple simulation ;
; ------------------------------------------------------- ;
;                                                         ;
;=========================================================;


;First things first: include miniODE's Blitz3D component in the project
Include "miniODE.bb"


;This needs to come before CreateSimulation
Graphics3D 1024,768,32,6

;Setup the Blitz3D world

Local SC_FPS=60		;Desired framerate - this is for "CPU breathing code"
Local VP_FPS=50		;Desired update rate - this mustn't change during the simulation
Local rtime=Floor(1000.0/SC_FPS),limited=True	;These variables all help with framerate control
Local phystime#=1000.0/VP_FPS
Local ctime=MilliSecs(),accum#=100,dtime

Local centre=CreatePivot()
Local camera = CreateCamera(centre)	;So we can see things happening!
PositionEntity camera,45,30,55
PointEntity camera,centre	;Point it at origin
Local light = CreateLight()
PositionEntity light,300,400,150
PointEntity light,centre


;Create a simulation and set the world's default gravity
ODE_CreateSimulation(0,-9.8,0)


;Let's create a floor

;First the Blitz entity
Local gmesh = CreateCube()
ScaleMesh gmesh,40,1,40
EntityColor gmesh,150,150,150

;Then the ODE body...
Local gbase=ODE_CreateBody()	;Create it like you would a mesh
ODE_SetMode gbase,2			;This sets the body's mode to "kinematic" - it will be completely static and not affected by forces
Local gplane=ODE_AddBox(gbase,80,2,80)	;Add a box geom, like you would a surface to a mesh
;The parameters are the box's actual size - since a default B3D cube is 2x2x2 and we scaled it 40x1x40, this is the same size

;We don't need to set the Blitz entity to the body if it isn't going to move, but let's do so for consistency
ODE_SetEntity gbase,gmesh


;Create some dynamic objects and let the fun begin...
Local i,bMesh,body
For i=1 To 50
	bMesh=CreateCube()	;Create a cube mesh
	EntityColor bMesh,Rand(0,16)*16,Rand(0,16)*16,Rand(0,16)*16
	
	body=ODE_CreateBody()	;Create a new ODE body
	ODE_SetPosition body,Rnd(-20,20),Rnd(10,40),Rnd(-20,20)				;Put it somewhere
	ODE_SetRotation body,Rand(-90,90),Rand(-180,180),Rand(-180,180)		;Make things interesting
	ODE_AddBox(body,2,2,2)	;Add a default-cube-sized collision box to the new body
	
	ODE_DefaultAutoDisable\flag=False
	ODE_SetAutoDisable body,ODE_DefaultAutoDisable
	
	ODE_SetEntity body,bMesh	;Set the B3D mesh to match the body's position from now on
Next


Local obj.ODE_Object	;ODE_Object is the type that holds body and entity data - don't worry, it's created automatically


While Not KeyDown(1)
	dtime=(MilliSecs()-ctime)			;Time measurement (for frame rate control)
	accum=accum+dtime
	ctime=MilliSecs()
	
	While accum>=phystime	;This maintains constant interface speed regardless of video frame rate
		CaptureWorld		;Putting movement commands in this loop will mean they are executed at a constant rate
							;This is pretty much essential for a stable ODE simulation
		
		;Apply forces and manipulate the physics here, if necessary
		
		If KeyHit(57)		;Let's make things more interesting
			For obj=Each ODE_Object
				If ODE_GetMode(obj\ODEbody)<>2	;Make sure it's not the static ground object
					ODE_SetMode obj\ODEbody,1	;Activate it (in case it's been auto-disabled)
					ODE_ApplyForce obj\ODEbody,Rnd(-50,50),Rnd(200,600),Rnd(-50,50)	;Random linear force
					ODE_ApplyTorque obj\ODEbody,Rnd(-50,50),Rnd(-50,50),Rnd(-50,50)	;Random spin
				EndIf
			Next
		EndIf
		
		ODE_UpdatePhysics 0.05		;Actually update the physics simulation
		
		;If you're also using Blitz or ColDet collision detection you'll want to update the Blitz3D world here
		
		;UpdateWorld goes here, if you need it
		
		accum=accum-phystime
	Wend
	
	;If you're only using Blitz3D graphically, you can update it in the render part of the loop
	ODE_UpdateScene
	
	RenderWorld accum/phystime		;Render a tweened value to maintain smooth visual updates
	
	Text 10,10,"Press space to kick things up a notch"
	
	Delay rtime-(MilliSecs()-ctime)-(limited+1)		;Free any spare CPU time
	Flip limited
Wend

;Free everything and exit
ODE_FreeSimulation
End

;~IDEal Editor Parameters:
;~C#Blitz3D