
;==============================;
;                              ;
; miniODE example 3: Materials ;
; ---------------------------- ;
;                              ;
;==============================;


;First things first: include miniODE's Blitz3D component in the project
Include "miniODE.bb"


;This needs to come before CreateSimulation
Graphics3D 1024,768,32,6

;Setup the Blitz3D world

Local SC_FPS=60		;Desired framerate - this is for "CPU breathing code"
Local VP_FPS=50		;Desired update rate - this mustn't change during the simulation
Local rtime=Floor(1000.0/SC_FPS),limited=True	;These variables all help with framerate control
Local phystime#=1000.0/VP_FPS
Local ctime=MilliSecs(),accum#=100,dtime

Local centre=CreatePivot()
Local camera = CreateCamera(centre)
PositionEntity camera,45,30,55
PointEntity camera,centre	;Point it at origin
Local light = CreateLight()
PositionEntity light,300,400,150
PointEntity light,centre


;Create a simulation and set the world's default gravity
ODE_CreateSimulation(0,-9.8,0)


;The floor, once more
Local gmesh = CreateCube()
ScaleMesh gmesh,40,1,40
EntityColor gmesh,150,150,150
Local gbase=ODE_CreateBody()
ODE_SetMode gbase,2
Local gplane=ODE_AddBox(gbase,80,2,80)
ODE_SetEntity gbase,gmesh
ODE_SetRotation gbase,20,0,0	;We're demonstrating friction...

;Our friends the cubes are back
Local box1=ODE_CreateBody()
ODE_AddBox(box1,10,10,10)
ODE_SetMass box1,5
ODE_SetPosition box1,20,40,-20
Local cube1=CreateCube()
ScaleEntity cube1,5,5,5
EntityColor cube1,255,0,0
ODE_SetEntity box1,cube1

Local box2=ODE_CreateBody()
ODE_AddBox(box2,10,10,10)
ODE_SetMass box2,5
ODE_SetPosition box2,0,40,-20
Local cube2=CreateCube()
ScaleEntity cube2,5,5,5
EntityColor cube2,0,0,255
ODE_SetEntity box2,cube2

Local box3=ODE_CreateBody()
ODE_AddBox(box3,10,10,10)
ODE_SetMass box3,5
ODE_SetPosition box3,-20,40,-20
Local cube3=CreateCube()
ScaleEntity cube3,5,5,5
EntityColor cube3,0,255,0
ODE_SetEntity box3,cube3


;Now to set up their materials...

;Materials are different from object and world parameters in that they aren't uploaded once
;by a Set call, but rather the Blitz type object is actually used by ODE as the source of
;material information, so you only need to set it once and remember not to delete it! You
;can also tweak values directly, but remember that this will affect every object with that
;material set (if it's the default, any new ones as well).

Local mat1.ODE_Material, mat2.ODE_Material, mat3.ODE_Material

mat1=ODE_CreateMaterial(ODE_CONTACT_BOUNCE,6.95,0.9,2)		;Stiff, bouncy
mat2=ODE_CreateMaterial(ODE_CONTACT_BOUNCE Or ODE_CONTACT_SOFTCFM,50,0.4,1,0.2,0.8)		;Jelly?
mat3=ODE_CreateMaterial()	;This one will start with just the defaults...

ODE_SetMaterial box1,mat1
ODE_SetMaterial box2,mat2
ODE_SetMaterial box3,mat3

;Onoes! The material is already set! But wait, we can change its fields anyway...
mat3\mode=ODE_CONTACT_BOUNCE	;You can use Or or + to combine modes
mat3\mu=0						;Friction - or not, in this case


;So where bodies are similar to entities and geoms are like surfaces, materials are meant to
;be intuitively similar to Blitz3D's Brush system. Just like brushes can be set on both
;whole entities and individual surfaces, ODE_SetMaterial can be called with either a whole
;rigid body, or an individual geom.

;Also remember that there are two partners in any collision and that therefore the actual
;effect you get will be a combination of the material settings for both surfaces.



While Not KeyDown(1)
	dtime=(MilliSecs()-ctime)			;Time measurement (for frame rate control)
	accum=accum+dtime
	ctime=MilliSecs()
	
	While accum>=phystime	;This maintains constant interface speed regardless of video frame rate
		CaptureWorld		;Putting movement commands in this loop will mean they are executed at a constant rate
							;This is pretty much essential for a stable ODE simulation
		
		;Apply forces and manipulate the physics here, if necessary
		
		ODE_UpdatePhysics 0.05		;Actually update the physics simulation
		
		;If you're also using Blitz or ColDet collision detection you'll want to update the Blitz3D world here
		
		;UpdateWorld goes here, if you need it
		
		accum=accum-phystime
	Wend
	
	;If you're only using Blitz3D graphically, you can update it in the render part of the loop
	ODE_UpdateScene
	
	RenderWorld accum/phystime		;Render a tweened value to maintain smooth visual updates
	
	Delay rtime-(MilliSecs()-ctime)-(limited+1)		;Free any spare CPU time
	Flip limited
Wend

;Free everything and exit
ODE_FreeSimulation
End

;~IDEal Editor Parameters:
;~C#Blitz3D