
;===========================;
;                           ;
; miniODE example 4: Joints ;
; ------------------------- ;
;                           ;
;===========================;


;First things first: include miniODE's Blitz3D component in the project
Include "miniODE.bb"


;This needs to come before CreateSimulation
Graphics3D 1024,768,32,6

;Setup the Blitz3D world

Local SC_FPS=60		;Desired framerate - this is for "CPU breathing code"
Local VP_FPS=50		;Desired update rate - this mustn't change during the simulation
Local rtime=Floor(1000.0/SC_FPS),limited=True	;These variables all help with framerate control
Local phystime#=1000.0/VP_FPS
Local ctime=MilliSecs(),accum#=100,dtime

Local centre=CreatePivot()
Local camera = CreateCamera(centre)
PositionEntity camera,45,30,55
PointEntity camera,centre	;Point it at origin
Local light = CreateLight()
PositionEntity light,300,400,150
PointEntity light,centre


;Create a simulation and set the world's default gravity
ODE_CreateSimulation(0,-9.8,0)


;The floor
Local gmesh = CreateCube()
ScaleMesh gmesh,40,1,40
EntityColor gmesh,150,150,150
Local gbase=ODE_CreateBody()
ODE_SetMode gbase,2
Local gplane=ODE_AddBox(gbase,80,2,80)
ODE_SetEntity gbase,gmesh

;We're going to need a target
Local i,bMesh,body
For i=1 To 8
	body=ODE_CreateBody()
	ODE_AddBox(body,6,5,6)
	ODE_SetPosition body,35,i*7.5,-35
	bMesh=CreateCube()
	ScaleMesh bMesh,3,2.5,3
	EntityColor bMesh,i*32,255-i*32,0
	ODE_SetEntity body,bMesh
Next

Local strut,arm,ball
;...and a catapult
strut=ODE_CreateBody()	;Support
ODE_AddCylinder(strut,10,20)
bMesh=CreateCylinder()
ScaleMesh bMesh,5,10,5
RotateMesh bMesh,90,0,0		;ODE cylinders and capsules are created at 90 degrees to their Blitz colleagues
EntityColor bMesh,0,0,255
ODE_SetPosition strut,-25,10,25
ODE_SetRotation strut,-90,0,0
ODE_SetEntity strut,bMesh
ODE_SetMode strut,2	;We need this to stay in place


arm=ODE_CreateBody()	;Firing arm
ODE_AddCapsule(arm,2,20)
bMesh=CreateCylinder()
ScaleMesh bMesh,2,10,2
RotateMesh bMesh,90,0,0
EntityColor bMesh,0,255,0
ODE_SetPosition arm,-25,25,25
ODE_SetEntity arm,bMesh

Local joint1, joint2, jointBroken
joint1 = ODE_CreateJoint(ODE_JOINT_HINGE,strut,arm)		;A hinge can go all the way around if not constrained
ODE_SetJointAxis joint1,0,1,0	;We want to spin around the Y axis


ball=ODE_CreateBody()	;Payload
ODE_AddSphere(ball,8)
bMesh=CreateSphere()
ScaleMesh bMesh,8,8,8
EntityColor bMesh,64,64,64
ODE_SetPosition ball,-25,25,5
ODE_SetEntity ball,bMesh

joint2 = ODE_CreateJoint(ODE_JOINT_SLIDER,arm,ball)		;A slider ought to keep it in line
ODE_LimitJoint joint2,1,0	;Constrain it rather a lot



While Not KeyDown(1)
	dtime=(MilliSecs()-ctime)			;Time measurement (for frame rate control)
	accum=accum+dtime
	ctime=MilliSecs()
	
	While accum>=phystime	;This maintains constant interface speed regardless of video frame rate
		CaptureWorld		;Putting movement commands in this loop will mean they are executed at a constant rate
							;This is pretty much essential for a stable ODE simulation
		
		;Apply forces and manipulate the physics here, if necessary
		If KeyDown(57) And ODE_GetAngularVelY(arm)<10
			ODE_SetMode arm,1
			ODE_ApplyTorque arm,0,1000-990*jointBroken,0
		EndIf
		
		If KeyHit(29) And jointBroken=0
			jointBroken=1
			ODE_FreeJoint joint2	;Fire!
		EndIf
		
		ODE_UpdatePhysics 0.05		;Actually update the physics simulation
		
		;If you're also using Blitz or ColDet collision detection you'll want to update the Blitz3D world here
		
		;UpdateWorld goes here, if you need it
		
		accum=accum-phystime
	Wend
	
	;If you're only using Blitz3D graphically, you can update it in the render part of the loop
	ODE_UpdateScene
	
	RenderWorld accum/phystime		;Render a tweened value to maintain smooth visual updates
	
	Text 10,10,"Hold space to spin up the firing arm"
	Text 10,30,"Press Ctrl to release the rock"
	
	Delay rtime-(MilliSecs()-ctime)-(limited+1)		;Free any spare CPU time
	Flip limited
Wend

;Free everything and exit
ODE_FreeSimulation
End

;~IDEal Editor Parameters:
;~C#Blitz3D